@@ -1,14 +1,14 @@
 from django.test import TestCase, Client, TransactionTestCase
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.contrib.auth.models import User
 from unittest import skip
 
 from .helpers import create_some_fake_parts, create_a_fake_organization, \
     create_a_fake_subpart, create_a_fake_partfile, \
     create_some_fake_part_classes, create_some_fake_manufacturers
-from .models import PartFile, Part
-from .forms import PartInfoForm, PartForm, AddSubpartForm
-from .octopart_parts_match import match_part
+from .models import PartFile, Part, SellerPart, ManufacturerPart, Seller
+from .forms import PartInfoForm, PartForm, AddSubpartForm, AddSellerPartForm
+from .octopart import match_part
 
 
 class TestBOM(TransactionTestCase):
@@ -24,11 +24,14 @@ class TestBOM(TransactionTestCase):
 
         (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
 
-        response = self.client.post(reverse('home'))
+        response = self.client.post(reverse('bom:home'))
+        self.assertEqual(response.status_code, 200)
+
+        response = self.client.get(reverse('bom:home'), {'q': p1.primary_manufacturer_part.manufacturer_part_number})
         self.assertEqual(response.status_code, 200)
 
     def test_error(self):
-        response = self.client.post(reverse('error'))
+        response = self.client.post(reverse('bom:error'))
         self.assertEqual(response.status_code, 200)
 
     def test_part_info(self):
@@ -38,14 +41,14 @@ class TestBOM(TransactionTestCase):
 
         response = self.client.post(
             reverse(
-                'part-info',
+                'bom:part-info',
                 kwargs={
                     'part_id': p1.id}))
         self.assertEqual(response.status_code, 200)
 
         response = self.client.post(
             reverse(
-                'part-info',
+                'bom:part-info',
                 kwargs={
                     'part_id': p2.id}))
         self.assertEqual(response.status_code, 200)
@@ -57,7 +60,7 @@ class TestBOM(TransactionTestCase):
 
         response = self.client.post(
             reverse(
-                'part-export-bom',
+                'bom:part-export-bom',
                 kwargs={
                     'part_id': p1.id}))
         self.assertEqual(response.status_code, 200)
@@ -66,7 +69,7 @@ class TestBOM(TransactionTestCase):
         (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
         with open('bom/test_parts.csv') as test_csv:
             response = self.client.post(
-                reverse('part-upload-bom', kwargs={'part_id': p1.id}),
+                reverse('bom:part-upload-bom', kwargs={'part_id': p1.id}),
                 {'file': test_csv})
         self.assertEqual(response.status_code, 302)
 
@@ -75,21 +78,21 @@ class TestBOM(TransactionTestCase):
 
         (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
 
-        response = self.client.post(reverse('export-part-list'))
+        response = self.client.post(reverse('bom:export-part-list'))
         self.assertEqual(response.status_code, 200)
 
-    @skip("only test when we want to hit octopart's api")
+    # @skip("only test when we want to hit octopart's api")
     def test_match_part(self):
         self.client.login(username='kasper', password='ghostpassword')
 
         (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
-        a = match_part(p1)
+        a = match_part(p1.primary_manufacturer_part, self.organization)
 
         partExists = len(a) > 0
 
         self.assertEqual(partExists, True)
 
-    @skip("only test when we want to hit octopart's api")
+    # @skip("only test when we want to hit octopart's api")
     def test_octopart_match_part_indented(self):
         self.client.login(username='kasper', password='ghostpassword')
 
@@ -97,12 +100,12 @@ class TestBOM(TransactionTestCase):
 
         response = self.client.post(
             reverse(
-                'part-octopart-match-bom',
+                'bom:part-octopart-match-bom',
                 kwargs={
                     'part_id': p1.id}))
         self.assertEqual(response.status_code, 302)
 
-    @skip("only test when we want to hit octopart's api")
+    # @skip("only test when we want to hit octopart's api")
     def test_part_octopart_match(self):
         self.client.login(username='kasper', password='ghostpassword')
 
@@ -110,18 +113,70 @@ class TestBOM(TransactionTestCase):
 
         response = self.client.post(
             reverse(
-                'part-octopart-match',
+                'bom:part-octopart-match',
                 kwargs={
                     'part_id': p1.id}))
         self.assertEqual(response.status_code, 302)
 
+    # @skip("only test when we want to hit octopart's api")
+    def test_manufacturer_part_octopart_match(self):
+        self.client.login(username='kasper', password='ghostpassword')
+
+        (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
+
+        response = self.client.post(
+            reverse(
+                'bom:manufacturer-part-octopart-match',
+                kwargs={
+                    'manufacturer_part_id': p1.primary_manufacturer_part.id}))
+        self.assertEqual(response.status_code, 302)
+
     def test_create_part(self):
         self.client.login(username='kasper', password='ghostpassword')
 
         (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
 
-        response = self.client.post(reverse('create-part'))
-        self.assertEqual(response.status_code, 200)
+        new_part_form_data = {
+            'manufacturer_part_number': 'STM32F401',
+            'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
+            'number_class': p1.number_class.id,
+            'number_item': '',
+            'number_variation': '',
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue('/part/' in response.url)
+
+        new_part_form_data = {
+            'manufacturer_part_number': 'STM32F401',
+            'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
+            'number_class': p1.number_class.id,
+            'number_item': '9999',
+            'number_variation': '01',
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue('/part/' in response.url)
+
+        new_part_form_data = {
+            'manufacturer_part_number': '',
+            'manufacturer': '',
+            'number_class': p1.number_class.id,
+            'number_item': '',
+            'number_variation': '',
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue('/part/' in response.url)
 
     def test_part_edit(self):
         self.client.login(username='kasper', password='ghostpassword')
@@ -130,7 +185,7 @@ class TestBOM(TransactionTestCase):
 
         response = self.client.post(
             reverse(
-                'part-edit',
+                'bom:part-edit',
                 kwargs={
                     'part_id': p1.id}))
         self.assertEqual(response.status_code, 200)
@@ -142,7 +197,7 @@ class TestBOM(TransactionTestCase):
 
         response = self.client.post(
             reverse(
-                'part-delete',
+                'bom:part-delete',
                 kwargs={
                     'part_id': p1.id}))
         self.assertEqual(response.status_code, 302)
@@ -154,7 +209,7 @@ class TestBOM(TransactionTestCase):
 
         response = self.client.post(
             reverse(
-                'part-add-subpart',
+                'bom:part-add-subpart',
                 kwargs={
                     'part_id': p1.id}))
         self.assertEqual(response.status_code, 302)
@@ -167,7 +222,7 @@ class TestBOM(TransactionTestCase):
 
         response = self.client.post(
             reverse(
-                'part-remove-subpart',
+                'bom:part-remove-subpart',
                 kwargs={
                     'part_id': p1.id,
                     'subpart_id': s1.id}))
@@ -180,7 +235,7 @@ class TestBOM(TransactionTestCase):
 
         response = self.client.post(
             reverse(
-                'part-remove-all-subparts',
+                'bom:part-remove-all-subparts',
                 kwargs={
                     'part_id': p1.id}))
         self.assertEqual(response.status_code, 302)
@@ -191,7 +246,7 @@ class TestBOM(TransactionTestCase):
         (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
         with open('bom/test_parts.csv') as test_csv:
             response = self.client.post(
-                reverse('part-upload-partfile', kwargs={'part_id': p1.id}),
+                reverse('bom:part-upload-partfile', kwargs={'part_id': p1.id}),
                 {'file': test_csv})
         self.assertEqual(response.status_code, 302)
 
@@ -199,7 +254,7 @@ class TestBOM(TransactionTestCase):
         for pf in partfiles:
             response = self.client.post(
                 reverse(
-                    'part-delete-partfile',
+                    'bom:part-delete-partfile',
                     kwargs={
                         'part_id': p1.id,
                         'partfile_id': pf.id}))
@@ -213,7 +268,7 @@ class TestBOM(TransactionTestCase):
             pf1 = create_a_fake_partfile(test_csv, p1)
             response = self.client.post(
                 reverse(
-                    'part-delete-partfile',
+                    'bom:part-delete-partfile',
                     kwargs={
                         'part_id': p1.id,
                         'partfile_id': pf1.id}))
@@ -224,7 +279,85 @@ class TestBOM(TransactionTestCase):
 
         with open('bom/test_new_parts.csv') as test_csv:
             response = self.client.post(
-                reverse('upload-parts'), {'file': test_csv})
+                reverse('bom:upload-parts'), {'file': test_csv})
+        self.assertEqual(response.status_code, 302)
+
+    def test_add_sellerpart(self):
+        self.client.login(username='kasper', password='ghostpassword')
+
+        (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
+
+        response = self.client.get(reverse('bom:manufacturer-part-add-sellerpart', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}))
+        self.assertEqual(response.status_code, 200)
+
+        response = self.client.post(reverse('bom:manufacturer-part-add-sellerpart', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}))
+        self.assertEqual(response.status_code, 200)
+
+        new_sellerpart_form_data = {
+            'seller': p1.optimal_seller().id,
+            'minimum_order_quantity': 1000,
+            'minimum_pack_quantity': 500,
+            'unit_cost': '1.23',
+            'lead_time_days': 25,
+            'nre_cost': 2000,
+            'ncnr': False,
+        }
+
+        response = self.client.post(reverse('bom:manufacturer-part-add-sellerpart', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}), new_sellerpart_form_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue('/part/' in response.url)
+
+    def test_sellerpart_delete(self):
+        self.client.login(username='kasper', password='ghostpassword')
+
+        (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
+        response = self.client.post(reverse('bom:sellerpart-delete', kwargs={'sellerpart_id': p1.optimal_seller().id}))
+
+        self.assertEqual(response.status_code, 302)
+
+    def test_manufacturer_part_edit(self):
+        self.client.login(username='kasper', password='ghostpassword')
+
+        (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
+        response = self.client.post(reverse('bom:manufacturer-part-edit', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}))
+        self.assertEqual(response.status_code, 200)
+
+        data = {
+            'manufacturer_part_number': 'ABC123',
+            'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
+            'name': '',
+        }
+
+        response = self.client.post(reverse('bom:manufacturer-part-edit', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}), data)
+        self.assertEqual(response.status_code, 302)
+
+        data = {
+            'manufacturer_part_number': 'ABC123',
+            'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
+            'name': 'A new manufacturer',
+        }
+
+        old_id = p1.primary_manufacturer_part.manufacturer.id
+        response = self.client.post(reverse('bom:manufacturer-part-edit', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}), data)
+        self.assertEqual(response.status_code, 302)
+        p1.refresh_from_db()
+        self.assertNotEqual(p1.primary_manufacturer_part.manufacturer.id, old_id)
+
+        data = {
+            'manufacturer_part_number': 'ABC123',
+            'manufacturer': '',
+            'name': '',
+        }
+
+        response = self.client.post(reverse('bom:manufacturer-part-edit', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}), data)
+        self.assertEqual(response.status_code, 200)  # 200 means it failed validation
+
+    def test_manufacturer_part_delete(self):
+        self.client.login(username='kasper', password='ghostpassword')
+
+        (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
+        response = self.client.post(reverse('bom:manufacturer-part-delete', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}))
+
         self.assertEqual(response.status_code, 302)
 
 
@@ -257,7 +390,7 @@ class TestForms(TestCase):
             'revision': 'AA'
         }
 
-        form = PartForm(organization=self.organization, data=form_data)
+        form = PartForm(data=form_data)
         self.assertTrue(form.is_valid())
 
         (m1, m2, m3) = create_some_fake_manufacturers(self.organization)
@@ -266,29 +399,25 @@ class TestForms(TestCase):
             'number_class': pc2.id,
             'description': "ASSY, ATLAS WRISTBAND 5",
             'revision': '1',
-            'manufacturer': m1.id,
         }
 
-        form = PartForm(organization=self.organization, data=form_data)
+        form = PartForm(data=form_data)
         self.assertTrue(form.is_valid())
-        
+
         new_part, created = Part.objects.get_or_create(
                 number_class=form.cleaned_data['number_class'],
                 number_item=form.cleaned_data['number_item'],
                 number_variation=form.cleaned_data['number_variation'],
-                manufacturer_part_number=form.cleaned_data['manufacturer_part_number'],
-                manufacturer=form.cleaned_data['manufacturer'],
                 organization=self.organization,
                 defaults={'description': form.cleaned_data['description'],
                           'revision': form.cleaned_data['revision'],
-                          }
-            )
+                          })
 
         self.assertTrue(created)
-        self.assertEqual(new_part.manufacturer, m1)
+        self.assertEqual(new_part.number_class.id, pc2.id)
 
     def test_part_form_blank(self):
-        form = PartForm({}, organization=self.organization)
+        form = PartForm({})
         self.assertFalse(form.is_valid())
         self.assertEqual(form.errors, {
             'number_class': [u'This field is required.'],
@@ -310,3 +439,23 @@ class TestForms(TestCase):
             'assembly_subpart': [u'This field is required.'],
             'count': [u'This field is required.'],
         })
+
+    def test_add_sellerpart_form(self):
+        (p1, p2, p3) = create_some_fake_parts(organization=self.organization)
+        form = AddSellerPartForm()
+        self.assertFalse(form.is_valid())
+
+        seller = Seller.objects.filter(organization=self.organization)[0]
+
+        form_data = {
+            'seller': seller.id,
+            'minimum_order_quantity': 1000,
+            'minimum_pack_quantity': 100,
+            'unit_cost': 1.2332,
+            'lead_time_days': 14,
+            'nre_cost': 1000,
+            'ncnr': True,
+        }
+
+        form = AddSellerPartForm(organization=self.organization, data=form_data)
+        self.assertTrue(form.is_valid())
